@@ -1,7 +1,8 @@
-
 import execnet
+
 gw = execnet.makegateway("popen//python=python2")
-channel = gw.remote_exec("""
+channel = gw.remote_exec(
+    """
     import numpy
     array = numpy.array([1,2,3])
     while 1:
@@ -10,8 +11,9 @@ channel = gw.remote_exec("""
             break
         array = numpy.append(array, x)
     channel.send(repr(array))
-""")
+"""
+)
 for x in range(10):
     channel.send(x)
 channel.send(None)
-print (channel.receive())
+print(channel.receive())
