@@ -8,22 +8,29 @@ showcasing features of the channel object:
 
 """
 import execnet
+
 gw = execnet.makegateway()
 
-outchan = gw.remote_exec("""
+outchan = gw.remote_exec(
+    """
     import sys
     outchan = channel.gateway.newchannel()
     sys.stdout = outchan.makefile("w")
     channel.send(outchan)
-""").receive()
+"""
+).receive()
+
 
 # note: callbacks execute in receiver thread!
 def write(data):
-    print "received:", repr(data)
+    print("received:", repr(data))
+
+
 outchan.setcallback(write)
 
-gw.remote_exec("""
-    print 'hello world'
-    print 'remote execution ends'
-""").waitclose()
-
+gw.remote_exec(
+    """
+    print('hello world')
+    print('remote execution ends')
+"""
+).waitclose()
