@@ -1,34 +1,33 @@
 #!/usr/bin/env python
-
 """
 
 small utility for hot-syncing a svn repository through ssh.
 uses execnet.
 
 """
-from __future__ import print_function
-import execnet
-import py
-import sys
 import os
+import pathlib
+import subprocess
+import sys
+
+import execnet
 
 
 def usage():
     arg0 = sys.argv[0]
-    print(arg0,
-          "[user@]remote-host:/repo/location localrepo [ssh-config-file]")
+    print(arg0, "[user@]remote-host:/repo/location localrepo [ssh-config-file]")
 
 
 def main(args):
     remote = args[0]
-    localrepo = py.path.local(args[1])
-    if not localrepo.check(dir=1):
-        raise SystemExit("localrepo {} does not exist".format(localrepo))
+    localrepo = pathlib.Path(args[1])
+    if not localrepo.is_dir():
+        raise SystemExit(f"localrepo {localrepo} does not exist")
     if len(args) == 3:
         configfile = args[2]
     else:
         configfile = None
-    remote_host, path = remote.split(':', 1)
+    remote_host, path = remote.split(":", 1)
     print("ssh-connecting to", remote_host)
     gw = getgateway(remote_host, configfile)
 
@@ -39,13 +38,20 @@ def main(args):
     # 2. server checks for newer revisions and sends dumps
     # 3. client receives dumps, updates local repo
     # 4. client goes back to step 1
-    c = gw.remote_exec("""
-        import py
+    c = gw.remote_exec(
+        """
         import os
+        import subprocess
         import time
+
         remote_rev, repopath = channel.receive()
-        while 1:
-            rev = py.process.cmdexec('svnlook youngest "%s"' % repopath)
+        while True:
+            rev = subprocess.run(
+                ["svnlook", "youngest", repopath],
+                check=True,
+                capture_output=True,
+                text=True,
+            ).stdout
             rev = int(rev)
             if rev > remote_rev:
                 revrange = (remote_rev+1, rev)
@@ -75,7 +81,8 @@ def main(args):
             else:
                 # using svn-hook instead would be nice here
                 time.sleep(30)
-    """)
+    """
+    )
 
     c.send((local_rev, path))
     print("checking revisions from %d in %s" % (local_rev, remote))
@@ -93,7 +100,7 @@ def svn_load(repo, dumpchannel, maxcount=100):
     # growing buffers  (execnet does not control
     # RAM usage or receive queue sizes)
     dumpchannel.send(maxcount)
-    f = os.popen("svnadmin load -q {}".format(repo), "w")
+    f = os.popen(f"svnadmin load -q {repo}", "w")
     count = maxcount
     for x in dumpchannel:
         sys.stdout.write(".")
@@ -103,12 +110,17 @@ def svn_load(repo, dumpchannel, maxcount=100):
         if count <= 0:
             dumpchannel.send(maxcount)
             count = maxcount
-    print >>sys.stdout
+    print()
     f.close()
 
 
 def get_svn_youngest(repo):
-    rev = py.process.cmdexec('svnlook youngest "%s"' % repo)
+    rev = subprocess.run(
+        ["svnlook", "youngest", repo],
+        check=True,
+        capture_output=True,
+        text=True,
+    ).stdout
     return int(rev)
 
 
@@ -118,7 +130,8 @@ def getgateway(host, configfile=None):
         xspec += "//ssh_config=%s" % configfile
     return execnet.makegateway(xspec)
 
-if __name__ == '__main__':
+
+if __name__ == "__main__":
     if len(sys.argv) < 3:
         usage()
         raise SystemExit(1)
