@@ -1,4 +1,3 @@
-# -*- coding: utf-8 -*-
 """
 sysinfo.py [host1] [host2] [options]
 
@@ -11,7 +10,6 @@ import re
 import sys
 
 import execnet
-import py
 
 
 parser = optparse.OptionParser(usage=__doc__)
@@ -35,14 +33,15 @@ parser.add_option(
 
 
 def parsehosts(path):
-    path = py.path.local(path)
+    host_regex = re.compile(r"Host\s*(\S+)")
     l = []
-    rex = re.compile(r"Host\s*(\S+)")
-    for line in path.readlines():
-        m = rex.match(line)
-        if m is not None:
-            (sshname,) = m.groups()
-            l.append(sshname)
+
+    with open(path) as fp:
+        for line in fp:
+            m = host_regex.match(line)
+            if m is not None:
+                (sshname,) = m.groups()
+                l.append(sshname)
     return l
 
 
@@ -119,7 +118,7 @@ class RemoteInfo:
 
 
 def debug(*args):
-    print >>sys.stderr, " ".join(map(str, args))
+    print(" ".join(map(str, args)), file=sys.stderr)
 
 
 def error(*args):
@@ -128,21 +127,21 @@ def error(*args):
 
 def getinfo(sshname, ssh_config=None, loginfo=sys.stdout):
     if ssh_config:
-        spec = "ssh=-F {} {}".format(ssh_config, sshname)
+        spec = f"ssh=-F {ssh_config} {sshname}"
     else:
         spec += "ssh=%s" % sshname
     debug("connecting to", repr(spec))
     try:
         gw = execnet.makegateway(spec)
-    except IOError:
+    except OSError:
         error("could not get sshgatway", sshname)
     else:
         ri = RemoteInfo(gw)
         # print "%s info:" % sshname
         prefix = sshname.upper() + " "
-        print >> loginfo, prefix, "fqdn:", ri.getfqdn()
+        print(prefix, "fqdn:", ri.getfqdn(), file=loginfo)
         for attr in ("sys.platform", "sys.version_info"):
-            loginfo.write("{} {}: ".format(prefix, attr))
+            loginfo.write(f"{prefix} {attr}: ")
             loginfo.flush()
             value = ri.getmodattr(attr)
             loginfo.write(str(value))
@@ -151,12 +150,12 @@ def getinfo(sshname, ssh_config=None, loginfo=sys.stdout):
         memswap = ri.getmemswap()
         if memswap:
             mem, swap = memswap
-            print >> loginfo, prefix, "Memory:", mem, "Swap:", swap
+            print(prefix, "Memory:", mem, "Swap:", swap, file=loginfo)
         cpuinfo = ri.getcpuinfo()
         if cpuinfo:
             numcpu, model = cpuinfo
-            print >> loginfo, prefix, "number of cpus:", numcpu
-            print >> loginfo, prefix, "cpu model", model
+            print(prefix, "number of cpus:", numcpu, file=loginfo)
+            print(prefix, "cpu model", model, file=loginfo)
         return ri
 
 
