@@ -1,4 +1,3 @@
-# -*- coding: utf-8 -*-
 """
     tests for multi channels and gateway Groups
 """
@@ -6,7 +5,6 @@ import gc
 from time import sleep
 
 import execnet
-import py
 import pytest
 from execnet import XSpec
 from execnet.gateway_base import Channel
@@ -141,7 +139,7 @@ class TestGroup:
     def test_group_default_spec(self):
         group = Group()
         group.defaultspec = "not-existing-type"
-        py.test.raises(ValueError, group.makegateway)
+        pytest.raises(ValueError, group.makegateway)
 
     def test_group_PopenGateway(self):
         group = Group()
@@ -176,7 +174,7 @@ class TestGroup:
         assert gw.id == "hello"
         gw = group.makegateway(specs[0])
         assert gw.id == "gw0"
-        # py.test.raises(ValueError,
+        # pytest.raises(ValueError,
         #    group.allocate_id, XSpec("popen//id=hello"))
         group.terminate()
 
